"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var sdk_const_1 = require("./sdk-const");
var client_data_1 = require("./client-data");
var utils_1 = require("./utils");
var befrest_message_1 = require("./befrest-message");
var befrest_connection_mode_1 = require("./befrest-connection-mode");
// @ts-ignore
var BefrestInternal = /** @class */ (function () {
    function BefrestInternal() {
        this.pingPayload = "";
        this.isBatchTypeReceived = false;
        this.pendingAcks = [];
        this.pendingMessages = [];
        this.isBefrestStart = false;
        this.acks = "";
        this.isBatchToSendSync = false;
    }
    BefrestInternal.prototype.start = function () {
        if (this.isBefrestStart) {
            console.log("Befrest-befrest already started");
            return;
        }
        this.isBefrestStart = true;
        console.log("Befrest-start");
        this.setupHeader();
        if (this.befrestWebSocket) {
            if (this.befrestWebSocket.readyState !== WebSocket.OPEN) {
                this.connectToServer();
            }
        }
        else {
            this.connectToServer();
        }
    };
    BefrestInternal.prototype.stop = function () {
        console.log("Befrest-stop");
        if (this.befrestWebSocket) {
            this.befrestWebSocket.close();
        }
        if (this.pingInterval) {
            clearInterval(this.pingInterval);
        }
        if (this.sendingAcksInterval) {
            clearInterval(this.sendingAcksInterval);
        }
        this.isBefrestStart = false;
    };
    BefrestInternal.prototype.refresh = function () {
        console.log("Befrest-refresh");
        if (this.befrestWebSocket == null || this.befrestWebSocket.readyState != WebSocket.OPEN) {
            this.start();
        }
        else {
            this.befrestWebSocket.send(sdk_const_1.SdkConst.PING_PREFIX + utils_1.getRandomNumber());
            this.isPendingPing = true;
            this.setupPingTimout();
        }
    };
    BefrestInternal.prototype.connectToServer = function () {
        var _this = this;
        console.log("Befrest-Connecting to server...");
        this.befrestWebSocket = new WebSocket(BefrestInternal.generateUrl());
        this.scheduleRestart();
        // @ts-ignore
        this.befrestWebSocket.onopen = function () {
            if (client_data_1.ClientData.getInstance().messageHandler) {
                client_data_1.ClientData.getInstance().messageHandler.onBefrestConnect();
                client_data_1.ClientData.getInstance().messageHandler.onConnectionChanged(befrest_connection_mode_1.BefrestConnectionMode.CONNECTED);
                _this.startPinging();
            }
            else {
                throw Error('messageHandler can not be null');
            }
        };
        // @ts-ignore
        this.befrestWebSocket.onmessage = function (message) {
            if (client_data_1.ClientData.getInstance().messageHandler) {
                client_data_1.ClientData.getInstance().messageHandler.onConnectionRefreshed();
            }
            _this.cancelPingTimeout();
            if (_this.checkPong(message.data)) {
                return;
            }
            var befrestMessage = new befrest_message_1.BefrestMessage(message.data);
            if (!befrestMessage.corruptionFlag) {
                console.log("Befrest-message receive : " + message.toString());
                switch (befrestMessage.getType) {
                    case befrest_message_1.MsgType.BATCH:
                        console.log("Befrest-batch message receive with size :" + message.data);
                        _this.isBatchTypeReceived = true;
                        _this.batchSize = 0;
                        _this.pendingAcks = [];
                        if (befrestMessage.message)
                            _this.prepareToReceiveMessages(befrestMessage.message);
                        break;
                    case befrest_message_1.MsgType.NORMAL:
                    case befrest_message_1.MsgType.GROUP:
                    case befrest_message_1.MsgType.TOPIC:
                        if (befrestMessage.getAckMessage)
                            if ((_this.acks.length + befrestMessage.getAckMessage).length >= 2 * 1024) {
                                if (_this.befrestWebSocket)
                                    _this.befrestWebSocket.send(_this.acks.substring(0, _this.acks.length - 1));
                                _this.acks = "";
                            }
                        _this.acks += befrestMessage.getAckMessage;
                        clearTimeout(_this.sendAckTimeOut);
                        _this.acks += "&";
                        _this.sendAckTimeOut = setTimeout(function () {
                            _this.sendAcksV2();
                        }, 1000);
                        if (_this.isBatchTypeReceived) {
                            if (_this.batchSize)
                                if (_this.batchSize > 0) {
                                    _this.batchSize--;
                                    _this.pendingMessages.push(befrestMessage);
                                }
                            if (_this.batchSize === 0) {
                                _this.isBatchToSendSync = true;
                                client_data_1.ClientData.getInstance().messageHandler.onMessagesReceived(_this.pendingMessages);
                                _this.pendingMessages = [];
                                _this.isBatchTypeReceived = false;
                                clearTimeout(_this.sendAckTimeOut);
                                _this.sendAckTimeOut = setTimeout(function () {
                                    _this.sendAcksV2();
                                }, 1000);
                            }
                        }
                        else {
                            _this.pendingMessages.push(befrestMessage);
                            client_data_1.ClientData.getInstance().messageHandler.onMessagesReceived(_this.pendingMessages);
                            _this.pendingMessages = [];
                        }
                        break;
                }
            }
        };
        if (this.befrestWebSocket)
            this.befrestWebSocket.onclose = function (ev) {
                _this.isBefrestStart = false;
                console.log("Befrest-connection close with message : " + ev.toString());
                if (client_data_1.ClientData.getInstance().messageHandler) {
                    client_data_1.ClientData.getInstance().messageHandler.onConnectionChanged(befrest_connection_mode_1.BefrestConnectionMode.DISCONNECTED);
                    if (_this.isRefreshingConnection) {
                        _this.start();
                    }
                }
                else {
                    throw Error('messageHandler can not be null');
                }
            };
        if (this.befrestWebSocket)
            this.befrestWebSocket.onerror = function (e) {
                try {
                    _this.isBefrestStart = false;
                    console.log("Befrest-error happens with message : " + e.message.toString());
                    if (client_data_1.ClientData.getInstance().messageHandler) {
                        client_data_1.ClientData.getInstance().messageHandler.onConnectionChanged(befrest_connection_mode_1.BefrestConnectionMode.DISCONNECTED);
                        if (e.message.toString().indexOf("401 Unauthorized")) {
                            client_data_1.ClientData.getInstance().messageHandler.onAuthorizeProblem();
                        }
                    }
                }
                catch (e) {
                }
            };
    };
    BefrestInternal.prototype.commitTopics = function () {
        console.log("Befrest-commit topics");
        this.isRefreshingConnection = true;
        this.stop();
    };
    BefrestInternal.prototype.startPinging = function () {
        var _this = this;
        if (this.befrestWebSocket == null || this.befrestWebSocket.readyState != WebSocket.OPEN || this.isPendingPing) {
            this.befrestWebSocket = undefined;
            this.start();
            return;
        }
        console.log("Befrest-start pinging with 10 sec interval");
        this.pingInterval = setInterval(function () {
            _this.isPendingPing = true;
            _this.pingPayload = sdk_const_1.SdkConst.PING_PREFIX + utils_1.getRandomNumber();
            if (_this.befrestWebSocket)
                _this.befrestWebSocket.send(_this.pingPayload);
            console.log("ping server with payload", _this.pingPayload);
            _this.setupPingTimout();
        }, 10000);
    };
    BefrestInternal.prototype.setupHeader = function () {
        console.log("Befrest-setup header with auth token : " +
            client_data_1.ClientData.getInstance().authToken + " and topics : " +
            client_data_1.ClientData.getInstance().topics);
        sdk_const_1.SdkConst.OPTIONS = {
            headers: {
                "X-BF-AUTH": client_data_1.ClientData.getInstance().authToken,
                Connection: "Upgrade",
                Upgrade: "websocket",
                "Sec-WebSocket-Key": "sample",
                "Sec-WebSocket-Version": "12",
                "X-BF-TOPICS": client_data_1.ClientData.getInstance().topics
            }
        };
    };
    BefrestInternal.generateUrl = function () {
        var t = "TOPIC";
        if (client_data_1.ClientData.getInstance().topics.length == 0)
            t = "/TOPIC";
        return sdk_const_1.SdkConst.BASE_URL
            .replace("AID", client_data_1.ClientData.getInstance().aid)
            .replace("CHID", client_data_1.ClientData.getInstance().chid)
            .replace("AUTH", client_data_1.ClientData.getInstance().authToken)
            .replace(t, client_data_1.ClientData.getInstance().topics);
    };
    BefrestInternal.prototype.scheduleRestart = function () {
        var _this = this;
        console.log("Befrest-scheduled restart after try to connect");
        setTimeout(function () {
            if (!_this.befrestWebSocket || _this.befrestWebSocket.readyState !== WebSocket.OPEN) {
                if (client_data_1.ClientData.getInstance().messageHandler) {
                    client_data_1.ClientData.getInstance().messageHandler.onConnectionChanged(befrest_connection_mode_1.BefrestConnectionMode.RETRY);
                    _this.isBefrestStart = false;
                }
                _this.start();
            }
        }, 3000);
    };
    BefrestInternal.prototype.checkPong = function (data) {
        return data.toString().toUpperCase().startsWith(sdk_const_1.SdkConst.PING_PREFIX);
    };
    BefrestInternal.prototype.prepareToReceiveMessages = function (message) {
        this.batchSize = Number(message);
    };
    BefrestInternal.prototype.sendAcksV2 = function () {
        var _this = this;
        console.log("Befrest-send batch Acks " + this.acks);
        if (this.befrestWebSocket)
            if (this.befrestWebSocket.readyState === WebSocket.OPEN) {
                this.befrestWebSocket.send(this.acks.substring(0, this.acks.length - 1));
                this.acks = "";
            }
        if (this.isBatchToSendSync) {
            setTimeout(function () {
                if (_this.befrestWebSocket)
                    if (_this.befrestWebSocket.readyState === WebSocket.OPEN) {
                        _this.befrestWebSocket.send("S");
                    }
            }, 50);
            this.isBatchToSendSync = false;
        }
    };
    BefrestInternal.prototype.cancelPingTimeout = function () {
        this.isPendingPing = false;
    };
    BefrestInternal.prototype.setupPingTimout = function () {
        var _this = this;
        setTimeout(function () {
            if (!_this.befrestWebSocket || _this.befrestWebSocket.readyState != WebSocket.OPEN || _this.isPendingPing) {
                if (client_data_1.ClientData.getInstance().messageHandler) {
                    client_data_1.ClientData.getInstance().messageHandler.onConnectionChanged(befrest_connection_mode_1.BefrestConnectionMode.RETRY);
                    _this.start();
                }
            }
        }, 3000);
    };
    return BefrestInternal;
}());
exports.BefrestInternal = BefrestInternal;
