"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var befrest_internal_1 = require("./befrest-internal");
var client_data_1 = require("./client-data");
var Befrest = /** @class */ (function () {
    function Befrest() {
        this.befrestInternal = new befrest_internal_1.BefrestInternal();
    }
    Befrest.getInstance = function () {
        if (!Befrest.instance) {
            Befrest.instance = new Befrest();
        }
        return Befrest.instance;
    };
    /**
     *init befrest with aid , chid  ,authToken and messageHandler
     * call this method first to use befrest
     */
    Befrest.prototype.init = function (aid, chid, authToken, messageHandler) {
        if (!aid.toString().trim().length) {
            throw Error("aid cannot be null");
        }
        if (!chid.toString().trim().length) {
            throw Error("chid cannot be null");
        }
        if (!messageHandler) {
            throw Error("messageHandler cannot be null");
        }
        if (aid !== client_data_1.ClientData.getInstance().aid) {
            client_data_1.ClientData.getInstance().aid = aid;
        }
        if (chid !== client_data_1.ClientData.getInstance().chid) {
            client_data_1.ClientData.getInstance().chid = chid;
        }
        if (authToken && authToken !== client_data_1.ClientData.getInstance().authToken) {
            client_data_1.ClientData.getInstance().authToken = authToken;
        }
        client_data_1.ClientData.getInstance().messageHandler = messageHandler;
    };
    /**
     *The befrest start method create connection to server to receive and send messages through websocket
     *call this method after init to start befrest
     */
    Befrest.prototype.start = function () {
        this.addTopic('x');
        this.befrestInternal.start();
    };
    /**
     * The befrest stop method disconnect from server after calling this method no message receive from befrest
     * call this method whenever you want to stop befrest
     */
    Befrest.prototype.stop = function () {
        this.befrestInternal.stop();
    };
    /**
     * The commitTopic method will be stop befrest and start again to update topic list in server
     * call this method whenever you add or remove topic(recommended)
     */
    Befrest.prototype.commitTopic = function () {
        this.befrestInternal.commitTopics();
    };
    /**
     * The refresh method send ping payload to server whenever you call to check connection status
     * and notifying you in onConnectionRefresh callback.
     */
    Befrest.prototype.refresh = function () {
        this.befrestInternal.refresh();
    };
    /**
     * The getTopicList method return a string array that show current topics that your client subscribed
     */
    Befrest.prototype.getTopicList = function () {
        return client_data_1.ClientData.getInstance().topics ? client_data_1.ClientData.getInstance().topics.split("-") : [];
    };
    /**
     * The addTopic method use to add specific topic
     * @param topic is a topic that you want to add
     * @return true if new topic added successfully and false if your topic has already exist
     */
    Befrest.prototype.addTopic = function (topic) {
        if (!topic.match(/^[a-z0-9]+$/i)) {
            console.log("topic name must be alpha-numeric");
            return false;
        }
        var addedTopic = topic.trim();
        if (client_data_1.ClientData.getInstance().topics.length === 0) {
            client_data_1.ClientData.getInstance().topics = addedTopic;
            return true;
        }
        else if (client_data_1.ClientData.getInstance().topics.toString().indexOf(addedTopic) === -1) {
            client_data_1.ClientData.getInstance().topics += "-" + addedTopic;
            return true;
        }
        else {
            console.log("Befrest-topic: " + topic + " has already exist int topic list");
            return false;
        }
    };
    /**
     * The removeTopic method use to remove specific topic
     * @param topic is a topic that you want to remove
     * @return true if topic remove successfully and false if topic not exists in topic list
     */
    Befrest.prototype.removeTopic = function (topic) {
        if (!topic.match(/^[a-z0-9]+$/i)) {
            console.log("topic name must be alpha-numeric");
            return false;
        }
        var toRemove = topic.trim();
        if (client_data_1.ClientData.getInstance().topics.length == 0) {
            console.log("Befrest-topic list is empty");
            return false;
        }
        else if (client_data_1.ClientData.getInstance().topics.toString().indexOf(toRemove) === -1) {
            console.log("Befrest-topic: " + toRemove + " not exist in topic list");
            return false;
        }
        client_data_1.ClientData.getInstance().topics = client_data_1.ClientData.getInstance().topics.replace(topic + "-", "");
        client_data_1.ClientData.getInstance().topics = client_data_1.ClientData.getInstance().topics.replace("-" + topic, "");
        return true;
    };
    return Befrest;
}());
exports.Befrest = Befrest;
