import {b64DecodeUnicode} from './utils';

export enum MsgType {
    NORMAL, BATCH, TOPIC, GROUP
}

interface MessagePattern {
    ts?: string;
    t?: number;
    mid?: string | undefined;
    m?: string;
}

export class BefrestMessage {
    private readonly messagePattern: MessagePattern;
    private messageType?: MsgType;
    private readonly data?: string;
    private readonly messageId?: string;
    private timeStamp?: string;
    private isCorrupted = false;

    constructor(data: string) {
        this.messagePattern = JSON.parse(data);
        if(!this.messagePattern.t || !this.messagePattern.ts || !this.messagePattern.m) {
            this.isCorrupted = true;
        }
        else {
            this.data = b64DecodeUnicode(this.messagePattern.m);
            this.timeStamp = this.messagePattern.ts;
            this.messageId = this.messagePattern.mid;
            this.setMessageType();
        }
    }

    private setMessageType() {
        switch(this.messagePattern.t) {
            case 1:
                this.messageType = MsgType.NORMAL;
                break;
            case 2:
                this.messageType = MsgType.BATCH;
                break;
            case 3:
                this.messageType = MsgType.TOPIC;
                break;
            case 4:
                this.messageType = MsgType.GROUP;
                break;
            default:
                this.isCorrupted = true;
                break;
        }
    }

    public get message() {
        return this.data;
    }

    public get corruptionFlag() {
        return this.isCorrupted;
    }

    public get getType() {
        return this.messageType;
    }

    public get getAckMessage() {
        if (this.getMessageType) 
        return "A" + this.getMessageType.toString().toUpperCase().charAt(0) + this.messageId;
    }


    private get getMessageType() {
        switch(this.messagePattern.t) {
            case 1:
                return("NORMAL");
            case 2:
                return("BATCH");
            case 3:
                return("TOPIC");
            case 4:
                return("GROUP");
            default:
                this.isCorrupted = true;
                break;
        }
    }

}